require "prefabutil"

local assets =
{
	Asset("ANIM", "anim/w_pond.zip"),
    Asset("ANIM", "anim/bee_box.zip"),
}

local prefabs =
{
	"fish",
	"eel",
	"froglegs",
	"slurtleslime",
} 

local levels = 
{
    { amount=3, idle="honey3", hit="hit_honey3" },
    { amount=2, idle="honey2", hit="hit_honey2" },
    { amount=1, idle="honey1", hit="hit_honey1" },
    { amount=0, idle="bees_loop", hit="hit_idle" },
}

local function StartSpawningFn(inst)
	local fn = function(world)
	if not inst:HasTag("burnt") then
	if inst.components.harvestable and inst.components.harvestable.growtime then
	inst.components.harvestable:StartGrowing()
end
	if inst.components.childspawner then
	inst.components.childspawner:StartSpawning()
		end
	end
end
	return fn
end

local function StopSpawningFn(inst)
	local fn = function(world)
	if inst.components.harvestable and inst.components.harvestable.growtime then
	inst.components.harvestable:StopGrowing()
end
	if inst.components.childspawner then
	inst.components.childspawner:StopSpawning()
	end
end
	return fn
end

local function onhammered(inst, worker)
	if inst:HasTag("fire") and inst.components.burnable then
	inst.components.burnable:Extinguish()
end
	inst.SoundEmitter:KillSound("loop")
	inst.components.lootdropper:DropLoot()
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function onhit(inst, worker)
	if not inst:HasTag("burnt") then
	inst.AnimState:PlayAnimation(inst.anims.hit)
	inst.AnimState:PushAnimation(inst.anims.idle, false)
	end
end

local function setlevel(inst, level)
    if not inst.anims then
    inst.anims = {idle = level.idle, hit = level.hit}
    else
    inst.anims.idle = level.idle
    inst.anims.hit = level.hit
end
    inst.AnimState:PlayAnimation(inst.anims.idle)
end

local function updatelevel(inst)
	if not inst:HasTag("burnt") then
	for k,v in pairs(levels) do
	if inst.components.harvestable.produce >= v.amount then
	setlevel(inst, v)
	            break
	        end
	    end
	end
end

local function onharvest(inst, picker)
	if not inst:HasTag("burnt") then
	updatelevel(inst)
	end
end

local function onchildgoinghome(inst, data)
	if not inst:HasTag("burnt") then
	if data.child and data.child.components.pollinator and data.child.components.pollinator:HasCollectedEnough() then
	if inst.components.harvestable then
	inst.components.harvestable:Grow()
	        end
	    end
	end
end

local function onsleep(inst)
	if not inst:HasTag("burnt") then
	if inst.components.harvestable then
	inst.components.harvestable:SetGrowTime(TUNING.WINTER_HARVEST_TIME)
	inst.components.harvestable:StartGrowing()
	    end
	end
end

local function stopsleep(inst)
	if not inst:HasTag("burnt") then
	if inst.components.harvestable then
	inst.components.harvestable:SetGrowTime(nil)
	inst.components.harvestable:StopGrowing()
	    end
	end
end

local function OnSave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
    data.burnt = true
    end
end

local function OnLoad(inst, data)
	if data and data.burnt then
    else
	updatelevel(inst)
	end
end

local function onbuilt(inst)
	inst.AnimState:PlayAnimation("place")
	inst.AnimState:PushAnimation("idle", false)
end

local function OnEntityWake(inst)
	if not inst:HasTag("burnt") then
    inst.SoundEmitter:PlaySound("dontstarve/bee/bee_box_LP", "loop")
    end
end

local function OnEntitySleep(inst)
	inst.SoundEmitter:KillSound("loop")
end

local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
	inst.entity:AddDynamicShadow()
	inst.DynamicShadow:SetSize( 4, 3 )

	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "w_pond.tex" )
	
    MakeObstaclePhysics(inst, .7)

	anim:SetBank("w_pond")
	anim:SetBuild("w_pond")
	anim:PlayAnimation("bees_loop")

    inst:AddTag("structure")
    inst:AddTag("playerowned")
    inst:AddTag("beebox")    
  
    inst:ListenForEvent("childgoinghome", onchildgoinghome)
   	inst:AddComponent("childspawner")
	inst.components.childspawner.childname = "bee"

	inst.lastseasonchecked="nothing"

	inst:DoPeriodicTask(1, function(inst)
	local season=GetWorld().components.seasonmanager.current_season
	if season ~= inst.lastseasonchecked and season == SEASONS.SPRING and inst.components.childspawner then
	inst.components.childspawner:SetRegenPeriod(math.ceil(TUNING.BEEBOX_REGEN_TIME / TUNING.SPRING_COMBAT_MOD))
	inst.components.childspawner:SetSpawnPeriod(math.ceil(TUNING.BEEBOX_RELEASE_TIME / TUNING.SPRING_COMBAT_MOD))
	inst.components.childspawner:SetMaxChildren(math.ceil(TUNING.BEEBOX_BEES * TUNING.SPRING_COMBAT_MOD))
	else
	if inst.components.childspawner then
	inst.components.childspawner:SetRegenPeriod(TUNING.BEEBOX_REGEN_TIME)
	inst.components.childspawner:SetSpawnPeriod(TUNING.BEEBOX_RELEASE_TIME)
	inst.components.childspawner:SetMaxChildren(TUNING.BEEBOX_BEES)
		end
	end
end)

	inst.components.childspawner:SetRegenPeriod( TUNING.BEEBOX_REGEN_TIME )
	inst.components.childspawner:SetSpawnPeriod( TUNING.BEEBOX_RELEASE_TIME )
	inst.components.childspawner:SetMaxChildren(TUNING.BEEBOX_BEES)

	inst.components.childspawner:StartSpawning()

	inst:ListenForEvent( "dusktime", StopSpawningFn(inst), GetWorld())
	inst:ListenForEvent( "daytime", StartSpawningFn(inst), GetWorld())

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = function(inst)
	if inst.components.harvestable and inst.components.harvestable:CanBeHarvested() then
	return "READY"
	end
end

    inst:AddComponent("harvestable")

    inst.components.harvestable:SetUp("fish", 3, nil, onharvest, updatelevel)

	inst:DoPeriodicTask(1, function(inst)
	local season=GetWorld().components.seasonmanager.current_season
	if season ~= inst.lastseasonchecked and season == SEASONS.WINTER then
	inst.components.harvestable.product="slurtleslime"
	if inst.components.harvestable.produce and inst.components.harvestable.produce>1 then
	inst.components.harvestable.produce=1
end
	inst.components.harvestable.maxproduce=1
	inst.lastseasonchecked=season
end
	if season ~= inst.lastseasonchecked and season == SEASONS.SUMMER then
	inst.components.harvestable.product="fish"
	inst.components.harvestable.maxproduce=3
	inst.lastseasonchecked=season
end
	if season ~= inst.lastseasonchecked and season == SEASONS.AUTUMN then
	inst.components.harvestable.product="froglegs"
	inst.components.harvestable.maxproduce=3
	inst.lastseasonchecked=season
end
	if season ~= inst.lastseasonchecked and season == SEASONS.SPRING then
	inst.components.harvestable.product="eel"
	inst.components.harvestable.maxproduce=3
	inst.lastseasonchecked=season
	end
end)

	inst:DoPeriodicTask(1, function(inst)
    if GetClock():IsDay() then
    inst.components.childspawner:StartSpawning()	
else
	inst.components.childspawner:StopSpawning()
	end
end)

    inst:AddComponent("lootdropper")

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)

	inst:ListenForEvent("entitysleep", onsleep)
	inst:ListenForEvent("entitywake", stopsleep)
	
    updatelevel(inst)

	inst.OnSave = OnSave 
	inst.OnLoad = OnLoad
	inst.OnEntitySleep = OnEntitySleep
	inst.OnEntityWake = OnEntityWake
	
	return inst
end

return Prefab("common/w_pond", fn, assets, prefabs),	
	   MakePlacer( "common/w_pond_placer", "w_pond", "w_pond", "bees_loop" ) 
